/*
 * Copyright (C) 2006 Vojtech Mencl
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup genericadt
 * @{
 */
/** @file
 */


#ifndef KERN_AVLTREE_H_
#define KERN_AVLTREE_H_ 

#include <arch/types.h>

/*
 * Makro for getting pointer to structure which enclosure avltree structure.
 *
 * @param link Pointer to avltree structure.
 * @param type Name of outer structure.
 * @param member Name of avltree atribute in outer structure.
 */
#define avltree_get_instance(link,type,member) \
	((type *)(((uint8_t *)(link)) - ((uint8_t *)&(((type *)NULL)->member))))


typedef struct avltree_node avltree_node_t;
typedef struct avltree avltree_t;


/** AVL tree node structure. */
struct avltree_node
{
	/** 
	 * Pointer to left descendand of this node.
	 *
	 * All keys of nodes in the left subtree are less then key of this node.
	 */
	struct avltree_node *lft;
	
	/** 
	 * Pointer to right descendand of this node.
	 *
	 * All keys of nodes in the right subtree are greater then key of this node.
	 */
	struct avltree_node *rgt;
	
	/** Pointer to parent node. Root node has NULL parent. */
	struct avltree_node *par;
	
	/** Nodes key. */
	uint64_t key; 
	
	/** Difference between heights of left and right subtree of this node.*/
	short balance;
};

/** AVL tree structure. */
struct avltree
{
	/** AVL root node pointer */
	struct avltree_node *root;

	/** 
	 * Base of tree is value that is smaller or equal then every value in tree.
	 *  
	 * Base is added to current key when new node is inserted into tree.
	 * Base is changed to the key of node which is deleted with function
	 *      avltree_delete_min.
	 */
	uint64_t base;             /**< POZOR: Base time for inserting new nodes */
};


/** Create empty AVL tree.
 *
 * @param t AVL tree.
 */
static inline void avltree_create (avltree_t *t)
{
	t->root = NULL;
	t->base = 0;
}

/** Initialize node.
 *
 * @param Node which is initialized.
 */
static inline void avltree_node_initialize(avltree_node_t *node)
{
	node->key = 0;
	node->lft = NULL;
	node->rgt = NULL;
	node->par = NULL;
	node->balance = 0;
}

avltree_node_t *avltree_find_min(avltree_t *t);
avltree_node_t *avltree_search(avltree_t *t, uint64_t key);
void avltree_insert(avltree_t *t, avltree_node_t *newnode);
void avltree_delete(avltree_t *t, avltree_node_t *node);
bool avltree_delete_min(avltree_t *t);

#endif

/** @}
 */
